<?php

namespace YottaPayIntegration\Payments\Api\Checkout;

/**
 * Class for request checkout type config field value
 */
class DescriptionType implements DescriptionTypeInterface
{
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_common = $common;
    }

    /**
     * Return checkout type
     *
     * @api
     * @return string
     */
    public function getType()
    {
        try
        {
            $this->_logger->debug('YottaPay | DescriptionType getType | Start');

            $checkoutType = $this->_common->getCheckoutDescriptionType();
            
            $loyaltyBonus = $this->_common->getParamLoyaltyBonus();

            if ($loyaltyBonus > 0)
            {
                $checkoutType = $checkoutType . '_loyalty';
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | DescriptionType getType | ' . $e->getMessage());

            //Default value
            $checkoutType = 'option_mobile_banking';
        }
        finally
        {
            $this->_logger->debug('YottaPay | DescriptionType getType | End');

            return $checkoutType;
        }
    }
}
