<?php

namespace YottaPayIntegration\Payments\Api\Payment\Old;

/**
 * Class for process payment result from Yotta Pay API
 */
class Result implements ResultInterface
{
    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\OrderSender
     */
    protected $_orderSender;

    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\InvoiceSender
     */
    protected $_invoiceSender;

    /**
     * @var \Magento\Sales\Model\Order\Payment\Transaction\Builder
     */
    protected $_transactionBuilder;

    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $_invoiceService;

     /**
     * @var \Magento\CatalogInventory\Api\StockManagementInterface
     */
    protected $_stockManagement;

    /**
     * @var \Magento\CatalogInventory\Model\Indexer\Stock\Processor
     */
    protected $_stockIndexerProcessor;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;
    
    /**
     * @var \Magento\Framework\DB\Transaction
     */
    protected $_dbTransaction;

    /**
     * @var \Magento\Framework\Notification\NotifierInterface
     */
    protected $_notifierPool;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;
    
    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender
     * @param \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder
     * @param \Magento\Sales\Model\Service\InvoiceService $invoiceService
     * @param \Magento\CatalogInventory\Api\StockManagementInterface $stockManagement
     * @param \Magento\CatalogInventory\Model\Indexer\Stock\Processor $stockIndexerProcessor
     * @param \Magento\Framework\DB\Transaction $dbTransaction
     * @param \Magento\Framework\Notification\NotifierInterface $notifierPool
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender,
        \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\CatalogInventory\Api\StockManagementInterface $stockManagement,
        \Magento\CatalogInventory\Model\Indexer\Stock\Processor $stockIndexerProcessor,
        \Magento\Framework\DB\Transaction $dbTransaction,
        \Magento\Framework\Notification\NotifierInterface $notifierPool,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_orderSender = $orderSender;
        $this->_invoiceSender = $invoiceSender;
        $this->_transactionBuilder = $transactionBuilder;
        $this->_invoiceService = $invoiceService;
        $this->_stockManagement = $stockManagement;
        $this->_stockIndexerProcessor = $stockIndexerProcessor;
        $this->_dbTransaction = $dbTransaction;
        $this->_notifierPool = $notifierPool;
        $this->_common = $common;
    }

    /**
     * Process payment result
     *
     * @param string $type
     * @param string $response_code
     * @param string $yottapay_transaction_identifier
     * @param string $shop_transaction_identifier
     * @param string $merchant_identifier
     * @param string $customer_identifier
     * @param string $amount
     * @param string $currency
     * @param string $signature
     * @return string
     */
    public function processPaymentResult(
        $type,
        $response_code,
        $yottapay_transaction_identifier,
        $shop_transaction_identifier,
        $merchant_identifier,
        $customer_identifier,
        $amount,
        $currency,
        $signature
    ) {
        try
        {
            $this->_logger->debug('YottaPay | processPaymentResult | Start');

            $this->_logger->debug('YottaPay | processPaymentResult | type:' . $type);
            $this->_logger->debug('YottaPay | processPaymentResult | response_code:' . $response_code);
            $this->_logger->debug('YottaPay | processPaymentResult | yottapay_transaction_identifier:' . $yottapay_transaction_identifier);
            $this->_logger->debug('YottaPay | processPaymentResult | shop_transaction_identifier:' . $shop_transaction_identifier);
            $this->_logger->debug('YottaPay | processPaymentResult | merchant_identifier:' . $merchant_identifier);
            $this->_logger->debug('YottaPay | processPaymentResult | customer_identifier:' . $customer_identifier);
            $this->_logger->debug('YottaPay | processPaymentResult | amount:' . $amount);
            $this->_logger->debug('YottaPay | processPaymentResult | currency:' . $currency);
            $this->_logger->debug('YottaPay | processPaymentResult | signature:' . $signature);

            $order = $this->checkRequestData(
                $type,
                $shop_transaction_identifier,
                $merchant_identifier,
                $customer_identifier,
                $amount,
                $currency
            );

            $this->checkRequestSign(
                $response_code,
                $yottapay_transaction_identifier,
                $shop_transaction_identifier,
                $merchant_identifier,
                $customer_identifier,
                $amount,
                $currency,
                $signature
            );

            $this->checkResponseCode($response_code, $yottapay_transaction_identifier, $order);

            $this->AddOldKeysNotice();

            return 'OK';
        }
        catch (\Exception $e)
        {
            $errorMessage = (
                'Error processPaymentResult Old [v. '
                . \YottaPayIntegration\Payments\Model\Common::$moduleVersion
                . ']: '
                . $e->getMessage()
            );

            $this->_logger->error('YottaPay | processPaymentResult Old | ' . $errorMessage);

            throw new \Magento\Framework\Webapi\Exception(
                __($errorMessage),
                0,
                \Magento\Framework\Webapi\Exception::HTTP_INTERNAL_ERROR
            );
        }
        finally
        {
            $this->_logger->debug('YottaPay | processPaymentResult Old | End');
        }
    }

    /**
     * Check required fields, then return the appropriate order
     *
     * @param string $type
     * @param string $shop_transaction_identifier
     * @param string $merchant_identifier
     * @param string $customer_identifier
     * @param string $amount
     * @param string $currency
     * @return \Magento\Sales\Model\Order
     */
    private function checkRequestData(
        $type,
        $shop_transaction_identifier,
        $merchant_identifier,
        $customer_identifier,
        $amount,
        $currency
    ) {
        if ($type != 'statuschange')
        {
            throw new \Exception('Invalid request data: type.');
        }

        $splited_shop_transaction_identifier = explode('-', $shop_transaction_identifier);
        $orderIncrementId = end($splited_shop_transaction_identifier);
        $order = $this->_common->getOrderByIncrementId($orderIncrementId);

        $temp_customer_identifier = (empty($order->getCustomerId()) ? '0' : $order->getCustomerId());

        $temp_shop_transaction_identifier = (
            \YottaPayIntegration\Payments\Model\Common::$platformPrefix
            . '-'
            .  $temp_customer_identifier
            . '-'
            . $order->getIncrementId()
        );

        $temp_merchant_identifier = $this->_common->getParamMerchantId();

        $temp_amount = number_format(round($order->getGrandTotal(), 2, PHP_ROUND_HALF_UP), 2, '.', '');

        $temp_currency = $order->getOrderCurrencyCode();

        if ($shop_transaction_identifier != $temp_shop_transaction_identifier)
        {
            throw new \Exception(
                'Invalid request data shop_transaction_identifier: '
                . $shop_transaction_identifier . ' and ' . $temp_shop_transaction_identifier
            );
        }

        if ($merchant_identifier != $temp_merchant_identifier)
        {
            throw new \Exception(
                'Invalid request data merchant_identifier: '
                . $merchant_identifier . ' and ' . $temp_merchant_identifier
            );
        }

        if ($customer_identifier != $temp_customer_identifier)
        {
            throw new \Exception(
                'Invalid request data customer_identifier: '
                . $customer_identifier . ' and ' . $temp_customer_identifier
            );
        }

        if ($amount != $temp_amount)
        {
            throw new \Exception(
                'Invalid request data amount: '
                . $amount . ' and ' . $temp_amount
            );
        }

        if ($currency != $temp_currency)
        {
            throw new \Exception(
                'Invalid request data currency: '
                . $currency . ' and ' . $temp_currency
            );
        }

        return $order;
    }

    /**
     * Uncancel order
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function uncancelOrder($order)
    {
        if ($order->getState() != 'canceled')
        {
            throw new \Exception(
                'Invalid order status for processing (#'
                . $order->getIncrementId()
                . '): "'
                . $order->getState()
                . '".'
            );
        }

        if ($order->isCanceled())
        {
            $productStockQty = [];

            foreach ($order->getAllVisibleItems() as $item)
            {
                $productStockQty[$item->getProductId()] = $item->getQtyCanceled();

                foreach ($item->getChildrenItems() as $child)
                {
                    $productStockQty[$child->getProductId()] = $item->getQtyCanceled();
                    $child->setQtyCanceled(0);
                    $child->setTaxCanceled(0);
                    $child->setDiscountTaxCompensationCanceled(0);
                }

                $item->setQtyCanceled(0);
                $item->setTaxCanceled(0);
                $item->setDiscountTaxCompensationCanceled(0);
            }

            $order->setSubtotalCanceled(0);
            $order->setBaseSubtotalCanceled(0);
            $order->setTaxCanceled(0);
            $order->setBaseTaxCanceled(0);
            $order->setShippingCanceled(0);
            $order->setBaseShippingCanceled(0);
            $order->setDiscountCanceled(0);
            $order->setBaseDiscountCanceled(0);
            $order->setTotalCanceled(0);
            $order->setBaseTotalCanceled(0);
            $order->setState("processing")->setStatus("processing");
            $order->addStatusHistoryComment(\YottaPayIntegration\Payments\Model\Common::$historyCommentOrderUncanceled)
                ->setIsCustomerNotified(true)
                ->save();

            /* Reverting inventory */
            $itemsForReindex = $this->_stockManagement->registerProductsSale(
                $productStockQty,
                $order->getStore()->getWebsiteId()
            );

            $productIds = [];

            foreach ($itemsForReindex as $item)
            {
                $item->save();
                $productIds[] = $item->getProductId();
            }

            if (!empty($productIds))
            {
                $this->_stockIndexerProcessor->reindexList($productIds);
            }

            $order->setInventoryProcessed(true);

            $order->save();
        }
        else
        {
            throw new \Exception('We cannot un-cancel this order.');
        }
    }

    /**
     * Caluclate signature and compare with requested value
     *
     * @param string $response_code
     * @param string $yottapay_transaction_identifier
     * @param string $shop_transaction_identifier
     * @param string $merchant_identifier
     * @param string $customer_identifier
     * @param string $amount
     * @param string $currency
     * @param string $signature
     * @return void
     */
    private function checkRequestSign(
        $response_code,
        $yottapay_transaction_identifier,
        $shop_transaction_identifier,
        $merchant_identifier,
        $customer_identifier,
        $amount,
        $currency,
        $signature
    ) {
        $merchantKey = $this->_common->getParamMerchantKey();

        $dataToSign = (
            $yottapay_transaction_identifier
            . $shop_transaction_identifier
            . $merchant_identifier
            . $customer_identifier
            . $amount
            . $currency
            . $response_code
            . $merchantKey
        );

        $calculatedSignature = hash('sha256', $dataToSign);

        if ($signature != $calculatedSignature)
        {
            $this->_logger->debug('YottaPay | checkRequestSign | Signature is wrong: ' . $signature . ' and ' . $calculatedSignature);

            throw new \Exception('Signature is wrong');
        }
    }

    /**
     * Send new order email
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function sendPaymentSuccessEmail($order)
    {
        if (!($order->getEmailSent()))
        {
            $order->setCanSendNewEmailFlag(true);
        }

        if ($order->getCanSendNewEmailFlag())
        {
            try
            {
                return $this->_orderSender->send($order);
            }
            catch (\Exception $e)
            {
                $this->_logger->error('YottaPay | sendPaymentSuccessEmail | ' . $errorMessage);

                return false;
            }
        }
    }

    /**
     * Send new invoice email
     *
     * @param \Magento\Sales\Model\Order\Invoice $invoice
     * @return void
     */
    private function sendInvoiceSuccessEmail($invoice)
    {
        if (!($invoice->getEmailSent()))
        {
            try
            {
                return $this->_invoiceSender->send($invoice);
            }
            catch (\Exception $e)
            {
                $this->_logger->error('YottaPay | sendInvoiceSuccessEmail | ' . $errorMessage);

                return false;
            }
        }
    }

    /**
     * Create transaction and add to payment
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $yottapay_transaction_identifier
     * @return void
     */
    private function addTransactionToPayment($order, $yottapay_transaction_identifier)
    {
        // Prepare payment object
        $paymentData = array();
        $payment = $order->getPayment();
        $payment->setLastTransId($yottapay_transaction_identifier);
        $payment->setTransactionId($yottapay_transaction_identifier);
        $payment->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]);
        $payment->setParentTransactionId(null);

        // Prepare transaction
        $transaction = $this->_transactionBuilder->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($yottapay_transaction_identifier)
            ->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData])
            ->setFailSafe(true)
            ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE);

        // Save payment and transaction
        $payment->save();
        $transaction->save();
    }

    /**
     * Change order state, send email and add history comment 
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $yottapay_transaction_identifier
     * @return void
     */
    private function processingOrder($order, $yottapay_transaction_identifier)
    {
        $order->setState("processing")->setStatus("processing");

        $this->sendPaymentSuccessEmail($order);

        $order->addStatusHistoryComment(
            \YottaPayIntegration\Payments\Model\Common::$historyCommentTransactionCompleted
            . $yottapay_transaction_identifier
        )
            ->setIsCustomerNotified(true)
            ->save();

        $order->save();
    }

    /**
     * Invoice order 
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function invoiceOrder($order)
    {
        $invoice = $this->_invoiceService->prepareInvoice($order);
        $invoice->register();
        $invoice->pay();
        $invoice->save();

        $transactionSave = $this->_dbTransaction->addObject($invoice)->addObject($invoice->getOrder());
        $transactionSave->save();

        $temp_total_amount = $order->getBaseCurrency()->formatTxt($order->getGrandTotal());

        $isNotified = $this->sendInvoiceSuccessEmail($invoice);

        $order->addStatusHistoryComment(
            \YottaPayIntegration\Payments\Model\Common::$historyCommentCaptured
            . $temp_total_amount
        )
            ->setIsCustomerNotified($isNotified)
            ->save();
            
        $order->save();
    }

    /**
     * Check operation result code from request and update order status
     *
     * @param string $response_code
     * @param string $yottapay_transaction_identifier
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function checkResponseCode($response_code, $yottapay_transaction_identifier, $order)
    {
        if ($response_code == '0')
        {
            //Uncancel order if long_term_confirmation param is enabled
            if ($order->getState() != 'pending_payment')
            {
                $long_term_confirmation = $this->_common->getParamLongTermConfirmation();

                if ($long_term_confirmation)
                {
                    $this->uncancelOrder($order);
                }
                else
                {
                    throw new \Exception(
                        'Invalid order status for processing (#'
                        . $order->getIncrementId()
                        . '): "'
                        . $order->getState()
                        . '".'
                    );
                }
            }

            // Prepare payment object
            $this->addTransactionToPayment($order, $yottapay_transaction_identifier);

            //Change order state and status
            $this->processingOrder($order, $yottapay_transaction_identifier);
            
            //Create Invoice if enabled
            if ($this->_common->getParamAutoInvoice() && $order->canInvoice())
            {
                $this->invoiceOrder($order);
            }
        }
        else
        {
            if ($order->getState() == 'pending_payment')
            {
                $order->cancel();
            }

            if ($response_code  == '2')
            {
                $historyComment = (
                    \YottaPayIntegration\Payments\Model\Common::$historyCommentTransactionCanceled
                    . $yottapay_transaction_identifier
                );
            }
            else
            {
                $historyComment = (
                    \YottaPayIntegration\Payments\Model\Common::$historyCommentTransactionFailed
                    . $yottapay_transaction_identifier
                );
            }

            $order->addStatusHistoryComment($historyComment)
                ->setIsCustomerNotified(true)
                ->save();

            $order->save();
        }
    }

    /**
     * Add admin notice about new API
     *
     * @return void
     */
    private function AddOldKeysNotice()
    {
        try
        {        
            $title = 'Yotta Pay now allows to use the new API';
            $message = 'Just go to Yotta Pay payment method config page and click "Log in with Yotta Pay".';
            $this->_notifierPool->addNotice($title, $message);
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | AddOldKeysNotice | ' . $e->getMessage());
        }
    }
}
