<?php

namespace YottaPayIntegration\Payments\Block\Adminhtml\Config;

/**
 * Renderer Login Button option field for Yotta Pay config
 */
class LoginButton extends \Magento\Config\Block\System\Config\Form\Field
{
    /**
     * @var string
     */
    protected $_template = 'YottaPayIntegration_Payments::config/login_button.phtml';

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $_request;

    /**
     * @var \Magento\Framework\View\Asset\Repository
     */
    protected $_assetRepo;

    /**
     * @var \Magento\Backend\App\Action\Context
     */
    protected $_backendContext;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\App\Action\Context $backendContext
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Framework\View\Asset\Repository $assetRepo
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param array $data
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\App\Action\Context $backendContext,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\View\Asset\Repository $assetRepo,
        \YottaPayIntegration\Payments\Model\Common $common,
        array $data = []
    ) {
        $this->_logger = $logger;
        $this->_request = $request;
        $this->_assetRepo = $assetRepo;
        $this->_backendContext = $backendContext;
        $this->_common = $common;

        parent::__construct($context, $data);
    }

    /**
     * Render element
     *
     * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $userToken = $this->_common->getUserToken();

        if (!empty($userToken))
        {
            return '<div id="payment-yottapay_gateway-loginbutton-hidden"></div>';
        }
        else
        {
            return parent::render($element);
        }
    }

    /**
     * Get element html
     *
     * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Retrieve url of a view file
     *
     * @param string $fileId
     * @param array $params
     * @return string
     */
    public function getViewFileUrl($fileId, array $params = [])
    {
        try
        {
            $params = array_merge(['_secure' => $this->_request->isSecure()], $params);
            return $this->_assetRepo->getUrlWithParams($fileId, $params);
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | getViewFileUrl | ' . $e->getMessage());
            return '';
        }
    }

    /**
     * Return admin authorize controller url
     *
     * @return string
     */
    public function getAuthorizeControllerUrl()
    {
        try
        {
            $backendUrl = $this->_backendContext->getUrl();
            $authorizeControllerUrl = $backendUrl->getUrl('yottapay/authorize/');

            return $authorizeControllerUrl;
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | getAuthorizeControllerUrl | ' . $e->getMessage());
            return '#';
        }
    }
}
