<?php

namespace YottaPayIntegration\Payments\Block;

use Magento\Framework\App\ObjectManager;
use Magento\Sales\Model\Rss\Signature;

/**
 * Yotta Pay loyalty popup
 */
class LoyaltyPopup extends \Magento\Framework\View\Element\Template
{
    /**
     * @var string
     */
    protected $_template = 'YottaPayIntegration_Payments::LoyaltyPopup.phtml';

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;
    
    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param array $data
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\View\Element\Template\Context $context,
        \YottaPayIntegration\Payments\Model\Common $common,
        array $data = []
    ) {
        $this->_logger = $logger;
        $this->_common = $common;

        parent::__construct($context, $data);
    }

    /**
     * Check if loyalty info should be displayed
     *
     * @return bool
     */
    public function isLoyaltyInfoAvailable()
    {
        try
        {
            $orderId = $this->_request->getParam('order_id');
            $order = $this->_common->getOrderById($orderId);

            if (!($this->_common->getParamActive()))
            {
                return false;
            }

            if ($this->_common->getParamLoyaltyBonus() == 0)
            {
                return false;
            }

            return $this->_common->validateOrderDateMatchLoyalty($order);
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | LoyaltyPopup isLoyaltyInfoAvailable | ' . $e->getMessage());
            return false;
        }
    }

    /**
     * Return to phtml Yotta Pay Gateway loyalty_bonus option value
     *
     * @return int
     */
    public function getLoyaltyBonusOption()
    {
        return $this->_common->getParamLoyaltyBonus();
    }
}
