<?php

namespace YottaPayIntegration\Payments\Block;

use Magento\Framework\App\ObjectManager;
use Magento\Sales\Model\Rss\Signature;

/**
 * Yotta Pay refund button
 */
class RefundButton extends \Magento\Framework\View\Element\Template
{
    /**
     * @var string
     */
    protected $_template = 'YottaPayIntegration_Payments::RefundButton.phtml';

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;
    
    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \YottaPayIntegration\Payments\Model\Common $common,
        array $data = []
    ) {
        $this->_common = $common;

        parent::__construct($context, $data);
    }

    /**
     * Get button url
     *
     * @return string
     */
    public function getLink()
    {
        $orderId = $this->_request->getParam('order_id');

        return $this->_common->getBaseUrl() . 'yottapay/refundform?order_id=' . $orderId;
    }

    /**
     * Get button label
     *
     * @return string
     */
    public function getLabel()
    {
        return 'Yotta Pay refund';
    }

    /**
     * Check if button should be displayed
     *
     * @return bool
     */
    public function isRefundButtonAvailable()
    {
        $orderId = $this->_request->getParam('order_id');
        $order = $this->_common->getOrderById($orderId);

        if ($order->getPayment()->getMethod() != 'yottapay_gateway' ||
            ($order->getState() != 'processing' && $order->getState() != 'complete'))
        {
            return false;
        }

        return true;
    }
}
