<?php

namespace YottaPayIntegration\Payments\Block;

/**
 * Yotta Pay refund form
 */
class RefundForm extends \Magento\Framework\View\Element\Template
{
    /**
     * @var string
     */
    protected $_template = 'YottaPayIntegration_Payments::customer/account/RefundForm.phtml';

    /**
     * @var Magento\Framework\Pricing\Helper\Data
     */
    protected $_pricingHelper;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Framework\Pricing\Helper\Data $pricingHelper
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Pricing\Helper\Data $pricingHelper,
        \YottaPayIntegration\Payments\Model\Common $common,
        array $data = []
    ) {
        $this->_pricingHelper = $pricingHelper;
        $this->_common = $common;

        parent::__construct($context, $data);
    }

    /**
     * Check order available
     *
     * @return bool
     */
    public function checkOrderAvailable()
    {
        try
        {
            $orderId = $this->_request->getParam('order_id');
            $order = $this->_common->getOrderById($orderId);

            return true;
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | RefundForm checkOrderAvailable | ' . $e->getMessage());

            return false;
        }
    }

    /**
     * Get order increment Id
     *
     * @return int
     */
    public function getOrderIncrementId()
    {
        $orderId = $this->_request->getParam('order_id');
        $order = $this->_common->getOrderById($orderId);

        return $order->getIncrementId();
    }

    /**
     * Get order total amount
     *
     * @return string
     */
    public function getOrderTotalAmount()
    {
        $orderId = $this->_request->getParam('order_id');
        $order = $this->_common->getOrderById($orderId);

        return $this->_pricingHelper->currency($order->getGrandTotal(), true, false);
    }
}
