<?php

namespace YottaPayIntegration\Payments\Controller\Adminhtml\Authorize;

/**
 * 
 */
class Index extends \Magento\Backend\App\Action
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Backend\App\Action\Context
     */
    protected $_backendContext;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Backend\App\Action\Context $backendContext
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Backend\App\Action\Context $context,
        \Magento\Backend\App\Action\Context $backendContext,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_backendContext = $backendContext;
        $this->_common = $common;

        return parent::__construct($context);
    }

    /**
     * Create authorize intent and redirect to responsed url
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     */
    public function execute()
    {
        try
        {
            $this->_logger->debug('YottaPay | Authorize execute | Start');

            //Initialize for response

            //No need to declare resultRedirectFactory in construct as its auto declared in construct of extended class
            $resultRedirect = $this->resultRedirectFactory->create();
            $backendUrl = $this->_backendContext->getUrl();
            $redirectUrl = $backendUrl->getUrl('admin/system_config/edit/section/payment/');

            //Create and save user_id
            $userId = $this->_common->getUuid();
            $this->_common->setUserId($userId);

            //Send authorize intent request to Yotta Pay API
            $responseData = $this->_common->performPostRequest(
                \YottaPayIntegration\Payments\Model\Common::$createAuthorizeIntentEndpoint,
                json_encode($this->getDataToSend()),
                'https://apiconnect.yottapay.co.uk'
            );

            //Process response data from Yotta Pay API
            $redirectUrl = $this->processResponseData($responseData);
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | Authorize execute | ' . $e->getMessage());
        }
        finally
        {
            $this->_logger->debug('YottaPay | Authorize execute | End');

            $resultRedirect->setUrl($redirectUrl);
            return $resultRedirect;
        }
    }

    /**
     * Check permission
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        //return $this->_authorization->isAllowed('Magento_Backend::all');
        return $this->_authorization->isAllowed('Magento_Sales::sales_creditmemo');
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @return array
     */
    private function processResponseData($responseData)
    {
        if (!array_key_exists('authUrl', $responseData))
        {
            throw new \Exception('processResponseData: Missing required parameters in Yotta Pay API response.');
        }

        return $responseData['authUrl'];
    }

    /**
     * Create data to authorize request
     *
     * @return array
     */
    private function getDataToSend()
    {
        $userId = $this->_common->getUserId();

        if (empty($userId))
        {
            throw new \Exception('Empty user_id config param.');
        }

        $baseUrl = $this->_common->getBaseUrl();

        $webhookUrl = $baseUrl . \YottaPayIntegration\Payments\Model\Common::$apiAuthorizeResult;
        $backendUrl = $this->_backendContext->getUrl();
        $redirectUrl = $backendUrl->getUrl('system_config/edit/section/payment/');

        $body = [
            'webhookUrl' => $webhookUrl,
            'redirectUrl' => $redirectUrl,
            'userId' => $userId,
        ];

        $this->_logger->debug('YottaPay | Authorize getDataToSend | body: ' . print_r($body, TRUE));

        return $body;
    }
}
