<?php

namespace YottaPayIntegration\Payments\Model;

/**
 * Common class
 */
class Common
{    
    public static $moduleVersion = "2.0.1";
    public static $platformPrefix = "M2";

    //Yotta Pay old API endpoints
    public static $createPaymentIntentEndpointOld = "launcher/shop/paymentgateway/new";
    public static $crateRefundIntentEndpointOld = "launcher/shop/paymentgateway/refund";
    public static $createDeferredEndpointOld = "launcher/shop/loyalty/create-deferred";
    public static $loyaltyIsEnabledEndpointOld = "launcher/shop/loyalty/is-enabled";

    //Old extension API endpoints
    public static $apiPaymentResultOld = "rest/all/V1/yottapay/payments/payment/old/result";
    public static $apiRefundResultOld = "rest/all/V1/yottapay/payments/refund/old/result";
    public static $apiLoyaltyResultOld = "rest/all/V1/yottapay/payments/loyalty/old/result";

    //Yotta Pay API endpoints
    public static $createAuthorizeIntentEndpoint = "/api/client/auth/new";
    public static $createPaymentIntentEndpoint = "api/client/order/new";
    public static $createDeferredEndpoint = "api/client/loyalty/create-deferred";
    public static $loyaltyIsEnabledEndpoint = "api/client/loyalty/is-active";
    public static $crateRefundIntentEndpoint = "api/client/refund/new";
    public static $stripeIsEnabledEndpoint = "/api/client/userdata/stripe-available";
    
    //Extension API endpoints
    public static $apiAuthorizeResult = "rest/all/V1/yottapay/payments/authorize/result";
    public static $apiPaymentResult = "rest/all/V1/yottapay/payments/payment/result";
    public static $apiPaymentCancellation = "rest/all/V1/yottapay/payments/cancellation/verification";
    public static $apiRefundResult = "rest/all/V1/yottapay/payments/refund/result";

    //Comments to order processing
    public static $historyCommentPaymentIntent = "Yotta Pay payment intent: ";
    public static $historyCommentOrderUncanceled = "Order uncanceled";
    public static $historyCommentCaptured = "Captured online amount of ";
    public static $historyCommentRefundRequested = "Yotta Pay refund requested";
    public static $historyCommentTransactionCompleted = "Yotta Pay transaction [completed]: ";
    public static $historyCommentTransactionCanceled = "Yotta Pay transaction [canceled]: ";
    public static $historyCommentTransactionFailed = "Yotta Pay transaction [failed]: ";

    //Authorization keys for requests to Yotta Pay API
    protected static $xClientKey = 'ec83a7c9-5685-40e1-b42e-2a57736ae438MGNT2';
    protected static $xClientSecret = '4e654534-f256-42fc-ba9f-084c771dc261MGNT2';

    protected static $connectTimeOut = 20;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    protected $_curl;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var \Magento\Framework\App\Config\Storage\WriterInterface
     */
    protected $_configWriter;

    /**
     * @var \Magento\Framework\App\Config\ReinitableConfigInterface
     */
    private $_reinitableConfig;

    /**
     * @var \Magento\Framework\Encryption\EncryptorInterface
     */
    private $_encryptor;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Sales\Model\Order
     */
    protected $_orderCollection;

    /**
     * @var \Magento\Sales\Model\OrderRepository
     */
    protected $_orderRepository;

    /**
     * @var int
     */
    protected $_storeId = null;

    /**
     * @var string
     */
    protected $_baseUrl = null;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\App\Config\Storage\WriterInterface $configWriter
     * @param \Magento\Framework\App\Config\ReinitableConfigInterface $reinitableConfig
     * @param \Magento\Framework\Encryption\EncryptorInterface $encryptor
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Sales\Model\Order $orderCollection
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
        \Magento\Framework\App\Config\ReinitableConfigInterface $reinitableConfig,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
        \Magento\Store\Model\StoreManagerInterface $storeManager,        
        \Magento\Sales\Model\Order $orderCollection,
        \Magento\Sales\Model\OrderRepository $orderRepository
    ) {
        $this->_logger = $logger;
        $this->_curl = $curl;
        $this->_scopeConfig = $scopeConfig;
        $this->_configWriter = $configWriter;
        $this->_reinitableConfig = $reinitableConfig;
        $this->_encryptor = $encryptor;
        $this->_storeManager = $storeManager;
        $this->_orderCollection = $orderCollection;
        $this->_orderRepository = $orderRepository;
    }

    /**
     * Return store id
     *
     * @return string
     */
    public function getStoreId()
    {
        if ($this->_storeId !== null) 
        {
            return $this->_storeId;
        }

        $this->_storeId = $this->_storeManager->getStore()->getId();

        return $this->_storeId;
    }

    /**
     * Return store base url
     *
     * @return string
     */
    public function getBaseUrl()
    {
        if ($this->_baseUrl !== null)
        {
            return $this->_baseUrl;
        }

        $this->_baseUrl = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);

        $isUseStore = (bool)((int)$this->_scopeConfig->getValue("web/url/use_store"));

        if ($isUseStore)
        {
            $storeCode = $this->_storeManager->getStore()->getCode();
            $this->_baseUrl = $this->_baseUrl . $storeCode . '/';
        }

        return $this->_baseUrl;
    }

    /**
     * Check sandbox parameter and return url
     *
     * @return string
     */
    public function getYottaPayUrl()
    {
        if (self::usedOldApiKeys())
        {
            $isSandbox = (bool)((int)$this->_scopeConfig->getValue(
                "payment/yottapay_gateway/sandbox",
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
                $this->getStoreId()
            ));

            if ($isSandbox)
            {
                return 'https://sandbox.yottapay.co.uk/';
            }
            else
            {
                return 'https://prod.yottapay.co.uk/';
            }
        }
        else
        {
            return 'https://apiconnect.yottapay.co.uk/';
        }
    }

    /**
     * Validates order currency
     *
     * @param \Magento\Sales\Model\Order $order
     * @return bool
     */
    public function validateOrderCurrency($order)
    {
        if ($order->getOrderCurrencyCode() != 'GBP')
        {
            return false;
        }

        return true;
    }
        
    /**
     * Validates billing country.
     *
     * @param \Magento\Sales\Model\Order $order
     * @return bool
     */
    public function validateBillingCountry($order)
    {
        if ($order->getBillingAddress()->getCountryId() != 'GB')
        {
            return false;
        }

        return true;
    }
        
    /**
     * Validates that the order meets the minimum order amount.
     *
     * @param \Magento\Sales\Model\Order $order
     * @return bool
     */
    public function validateMinOrderAmount($order)
    {
        if (number_format(round($order->getGrandTotal(), 2, PHP_ROUND_HALF_UP), 2, '.', '') < 0.01)
        {
            return false;
        }

        return true;
    }

    /**
     * Validate that the order creation date match with welcome loyalty program period.
     *
     * @param \Magento\Sales\Model\Order $order
     * @return bool
     */
    public function validateOrderDateMatchLoyalty($order)
    {
        $dateOrderCreated = strtotime($order->getCreatedAt());
        $dateLoyaltyStart = strtotime($this->getParamLoyaltyStartPeriod());
        
        if ($dateOrderCreated < $dateLoyaltyStart)
        {
            return false;
        }

        if ($order->getState() != 'processing' && $order->getState() != 'complete')
        {         
            return false;  
        }

        return true;
    }

    /**
     * Return active parameter value
     *
     * @return bool
     */
    public function getParamActive()
    {
        $pluginIsActive = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/active",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($pluginIsActive))
        {
            return (bool)((int)$pluginIsActive);
        }
        else
        {
            return false;
        }
    }

    /**
     * Return user_id parameter value
     *
     * @return string
     */
    public function getUserId()
    {
        $userId = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/user_id",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($userId))
        {
            return trim($userId);
        }
        else
        {
            return '';
        }        
    }

    /**
     * Set user_id parameter value
     */
    public function setUserId($userId)
    {
        $encryptedUserId = $this->_encryptor->encrypt($userId);

        $this->_configWriter->save(
            "payment/yottapay_gateway/user_id",
            $encryptedUserId
        );

        $this->_reinitableConfig->reinit();
    }

    /**
     * Return user_token parameter value
     *
     * @return string
     */
    public function getUserToken()
    {
        $userToken = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/user_token",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($userToken))
        {
            return trim($userToken);
        }
        else
        {
            return '';
        }
    }

    /**
     * Set user_token parameter value
     */
    public function setUserToken($userToken)
    {
        $encryptedUserToken = $this->_encryptor->encrypt($userToken);

        $this->_configWriter->save(
            "payment/yottapay_gateway/user_token",
            $encryptedUserToken
        );

        $this->_reinitableConfig->reinit();
    }

    /**
     * Return merchant_identifier parameter value
     *
     * @return string
     */
    public function getParamMerchantId()
    {
        $merchantId = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/merchant_identifier",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($merchantId))
        {
            return trim($merchantId);
        }
        else
        {
            return '';
        }
    }

    /**
     * Set merchant_identifier parameter value
     */
    public function setMerchantId($merchantId)
    {
        $this->_configWriter->save(
            "payment/yottapay_gateway/merchant_identifier",
            $merchantId
        );

        $this->_reinitableConfig->reinit();
    }

    /**
     * Return merchant_gateway_key parameter value
     *
     * @return string
     */
    public function getParamMerchantKey()
    {
        $merchantKey = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/merchant_gateway_key",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($merchantKey))
        {
            return trim($merchantKey);
        }
        else
        {
            return '';
        }
    }

    /**
     * Set merchant_gateway_key parameter value
     */
    public function setMerchantKey($merchantKey)
    {
        $this->_configWriter->save(
            "payment/yottapay_gateway/merchant_gateway_key",
            $merchantKey
        );

        $this->_reinitableConfig->reinit();
    }

    /**
     * Return auto_invoice parameter value
     *
     * @return bool
     */
    public function getParamAutoInvoice()
    {
        $autoInvoice = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/auto_invoice",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($autoInvoice))
        {
            return (bool)((int)$autoInvoice);
        }
        else
        {
            return false;
        }
    }

    /**
     * Return long_term_confirmation parameter value
     *
     * @return bool
     */
    public function getParamLongTermConfirmation()
    {
        $longTermConfirmation = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/long_term_confirmation",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($longTermConfirmation))
        {
            return (bool)((int)$longTermConfirmation);
        }
        else
        {
            return false;
        }
    }

    /**
     * Return checkout_description_type parameter value
     *
     * @return string
     */
    public function getCheckoutDescriptionType()
    {
        $type = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/checkout_description_type",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($type))
        {
            return trim($type);
        }
        else
        {
            return '';
        }
    }

    /**
     * Set checkout_description_type parameter value
     */
    public function setCheckoutDescriptionType($type)
    {
        $this->_configWriter->save(
            "payment/yottapay_gateway/checkout_description_type",
            $type
        );

        $this->_reinitableConfig->reinit();
    }

    /**
     * Return gtm_id parameter value
     *
     * @return string
     */
    public function getGtmId()
    {
        $gtmId = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/gtm_id",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($gtmId))
        {
            return trim($gtmId);
        }
        else
        {
            return '';
        }
    }

    /**
     * Return loyalty bonus parameter value
     *
     * @return int
     */
    public function getParamLoyaltyBonus()
    {
        $loyaltyBonus = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/loyalty_bonus",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($loyaltyBonus))
        {
            return (int)$loyaltyBonus;
        }
        else
        {
            return 0;
        }
    }

    /**
     * Return loyalty bonus max parameter value
     *
     * @return int
     */
    public function getParamLoyaltyBonusMax()
    {
        $loyaltyMaxBonus = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/loyalty_max_bonus",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($loyaltyMaxBonus))
        {
            return (int)$loyaltyMaxBonus;
        }
        else
        {
            return 0;
        }
    }

    /**
     * Return loyalty start period parameter value
     *
     * @return string
     */
    public function getParamLoyaltyStartPeriod()
    {
        $loyaltyStartPeriod = $this->_scopeConfig->getValue(
            "payment/yottapay_gateway/loyalty_start_period",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($loyaltyStartPeriod))
        {
            return strval($loyaltyStartPeriod);
        }
        else
        {
            return '';
        }
    }

    /**
     * Return email identity
     *
     * @return string
     */
    public function getParamEmailIdentity()
    {
        $emailIdentity = $this->_scopeConfig->getValue(
            "sales_email/order/identity",
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->getStoreId()
        );

        if (isset($emailIdentity))
        {
            return strval($emailIdentity);
        }
        else
        {
            return '';
        }
    }

    /**
     * Perform POST request
     *
     * @param string $requestEndpoint
     * @param string $requestBody
     * @param string $requestPath
     * @return array
     */
    public function performPostRequest($requestEndpoint, $requestBody, $requestPath = '')
    {
        if (empty($requestPath))
        {
            $requestUrl = $this->getYottaPayUrl() . $requestEndpoint;            
        }
        else
        {
            $requestUrl = $requestPath . $requestEndpoint;
        }

        $this->_curl->setOption(CURLOPT_CONNECTTIMEOUT, self::$connectTimeOut);
        $this->_curl->addHeader('Content-Type', 'application/json');
        $this->_curl->addHeader('X-CLIENT-KEY', self::$xClientKey);
        $this->_curl->addHeader('X-CLIENT-SECRET', self::$xClientSecret);


        $this->_logger->debug('YottaPay | performPostRequest | RequestUrl: ' . $requestUrl);
        $this->_logger->debug('YottaPay | performPostRequest | RequestData: ' . $requestBody);

        $this->_curl->post($requestUrl, $requestBody);

        $result = $this->_curl->getBody();
        $httpCode = $this->_curl->getStatus();
        
        if ($httpCode != 200)
        {
            throw new \Exception('Yotta Pay API response code: ' . $httpCode);
        }

        $responseData = json_decode($result, true);

        $this->_logger->debug('YottaPay | performPostRequest | HttpCode: ' . $httpCode);
        $this->_logger->debug('YottaPay | performPostRequest | ResponseData: ' . $result);

        return $responseData;
    }

    /**
     * Perform GET request
     *
     * @param string $requestEndpoint
     * @param string $requestPath
     * @return array
     */
    public function performGetRequest($requestEndpoint, $requestPath = '')
    {
        if (empty($requestPath))
        {
            $requestUrl = $this->getYottaPayUrl() . $requestEndpoint;            
        }
        else
        {
            $requestUrl = $requestPath . $requestEndpoint;
        }

        $this->_curl->setOption(CURLOPT_CONNECTTIMEOUT, self::$connectTimeOut);
        $this->_curl->addHeader('X-CLIENT-KEY', self::$xClientKey);
        $this->_curl->addHeader('X-CLIENT-SECRET', self::$xClientSecret);


        $this->_logger->debug('YottaPay | performGetRequest | RequestUrl: ' . $requestUrl);

        $this->_curl->get($requestUrl);

        $result = $this->_curl->getBody();
        $httpCode = $this->_curl->getStatus();
        
        if ($httpCode != 200)
        {
            throw new \Exception('Yotta Pay API response code: ' . $httpCode);
        }

        $responseData = json_decode($result, true);

        $this->_logger->debug('YottaPay | performGetRequest | HttpCode: ' . $httpCode);
        $this->_logger->debug('YottaPay | performGetRequest | ResponseData: ' . $result);

        return $responseData;
    }

    /**
     * Get data from request and return decoded json.
     *
     * @return array
     */
    public function getRequestData()
    {
        $body = file_get_contents('php://input');

        if (empty($body))
        {
            throw new \Exception('No data in request.');
        }

        $requestData = json_decode($body, true);

        $this->_logger->debug('YottaPay | getRequestData | RequestData: ' . $body);

        return $requestData;
    }

    /**
     * Return order by id
     *
     * @param int $orderIncrementId
     * @return \Magento\Sales\Model\Order $order
     */
    public function getOrderById($orderId)
    {
        try 
        {
            $order = $this->_orderRepository->get($orderId);

            if (empty($order) || empty($order->getState()))
            {
                throw new \Exception('Order with id ' . $orderId . ' not found.');
            }

            return $order;
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | getOrderById | ' . $e->getMessage());
            throw new \Exception('Order with id ' . $orderId . ' not found.');
        }
    }

    /**
     * Return order by increment id
     *
     * @param int $orderIncrementId
     * @return \Magento\Sales\Model\Order $order
     */
    public function getOrderByIncrementId($orderIncrementId)
    {
        try
        {
            $order = $this->_orderCollection->loadByIncrementId($orderIncrementId);

            if (empty($order) || empty($order->getState()))
            {
                throw new \Exception('Order with increment id #' . $orderIncrementId . 'not found.');
            }

            return $order;
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | getOrderByIncrementId | ' . $e->getMessage());
            throw new \Exception('Order with increment id #' . $orderIncrementId . 'not found.');
        }
    }

    /**
     * Verify customer and return clientUserId
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $merchantId
     * @return string
     */
    public function getClientUserId($order)
    {
        if (self::usedOldApiKeys())
        {
            $merchantId = $this->getParamMerchantId();

            $tempCustomerIdentifier = empty($order->getCustomerId()) ? '0' : $order->getCustomerId();

            if ($tempCustomerIdentifier != '0')
            {
                $clientUserId = $merchantId . '_1_' . $tempCustomerIdentifier; //if account
            }
            else
            {
                $clientUserId = $merchantId . '_0_' . $order->getId(); //If guest ()
            }
        }
        else
        {
            $tempCustomerIdentifier = (empty($order->getCustomerId()) ? '0' : $order->getCustomerId());

            if ($tempCustomerIdentifier != '0')
            {
                $clientUserId = '1_' . $tempCustomerIdentifier; //if account
            }
            else
            {
                $clientUserId = '0_' . $order->getId(); //If guest ()
            }
        }       

        return $clientUserId;
    }

    /**
     * Verify loyalty parameters and return amount
     *
     * @param \Magento\Sales\Model\Order $order
     * @return int
     */
    public function getLoyaltyAmount($order)
    {
        $loyaltyBonus = $this->getParamLoyaltyBonus();
        $loyaltyMaxBonus = $this->getParamLoyaltyBonusMax();

        $bonus = (($loyaltyBonus * $order->getGrandTotal()) / 100);
        $roundedBonus = intval(number_format(round($bonus, 2, PHP_ROUND_HALF_UP), 2, '', '')); //without point

        if ($roundedBonus < 1)
        {
            $roundedBonus = 1;
        }

        $roundedMaxBonus = intval(number_format(round($loyaltyMaxBonus, 2, PHP_ROUND_HALF_UP), 2, '', '')); //without point

        if (($roundedMaxBonus > 0) && ($roundedBonus > $roundedMaxBonus))
        {
            $loyaltyAmount = $roundedMaxBonus;
        }
        else
        {
            $loyaltyAmount = $roundedBonus;
        }

        return $loyaltyAmount;
    }

     /**
     * Return the usage status of old API keys
     *
     * @return bool
     */
    public function usedOldApiKeys()
    {
        $merchantId = self::getParamMerchantId();
        $merchantKey = self::getParamMerchantKey();

        if (!empty($merchantId) || !empty($merchantKey))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

     /**
     * Return new UUID
     *
     * @return string
     */
    public static function getUuid($lenght = 16)
    {
        if (function_exists("random_bytes"))
        {
            $bytes = random_bytes($lenght);
        }
        elseif (function_exists("openssl_random_pseudo_bytes"))
        {
            $bytes = openssl_random_pseudo_bytes($lenght);
        }
        else
        {
            return uniqid();
        }

        return bin2hex($bytes);
    }
}
