<?php

namespace YottaPayIntegration\Payments\Model\Config\Backend;

/**
 * 
 */
class GTMOption extends \Magento\Framework\App\Config\Value
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $config
     * @param \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \YottaPayIntegration\Payments\Model\Common $common,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\App\Config\ScopeConfigInterface $config,
        \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [] 
    ) {
        $this->_logger = $logger;
        $this->_messageManager = $messageManager;
        $this->_common = $common;

        parent::__construct($context, $registry, $config, $cacheTypeList, $resource, $resourceCollection, $data);
    }

    /**
     * Check value for according Google Tag Manager ID mask
     *
     * @return \Magento\Framework\Model\AbstractModel
     */
    public function beforeSave()
    {
        try
        {
            $this->_logger->debug('YottaPay | GTMOption beforeSave | Start');
            
            $value = $this->getValue();

            if (strlen($value) == 0)
            {
                return parent::beforeSave();
            }

            $pattern = '/^GTM-[A-Z0-9]{1,8}$/';
            $preg_result = preg_match($pattern, $value);

            if (!(boolval($preg_result) && wc_strtolower($preg_result) != 'false'))
            {
               throw new \Exception('Invalid Google Tag Manager Id.'); 
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | GTMOption beforeSave | ' . $e->getMessage());

            $errMsg = ('Please enter a valid Google Tag Manager Id or leave it blank.');
            $this->_messageManager->addError($errMsg);

            $this->setValue('');
        }
        finally
        {
            $this->_logger->debug('YottaPay | GTMOption beforeSave | End');
            return parent::beforeSave();
        }
    }
}
