<?php

namespace YottaPayIntegration\Payments\Observer;

/**
 * Yotta Pay module observer
 */
class CheckBackButton implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $_request;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Request\Http $request
     * @param \Magento\Checkout\Model\Session $checkoutSession
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Checkout\Model\Session $checkoutSession
    ) {
        $this->_logger = $logger;
        $this->_request = $request;
        $this->_checkoutSession = $checkoutSession;
    }

    /**
     * Restore previous quote
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        try
        {
            $lastRealOrder = $this->_checkoutSession->getLastRealOrder();        

            if (strpos($this->_request->getPathInfo(), '/checkout/onepage/success') !== false)
            {
                return;
            }

            if (strpos($this->_request->getPathInfo(), '/customer/section/load/') !== false &&
                $lastRealOrder->getId() &&
                $this->_checkoutSession->getQuoteId())
            {
                return;
            }

            if ($lastRealOrder->getPayment())
            {
                if ($lastRealOrder->getPayment()->getMethod() != 'yottapay_gateway')
                {
                    return;
                }

                if ($lastRealOrder->getPayment()->getLastTransId())
                {
                    return;
                }

                $historyItem = $lastRealOrder->getStatusHistoryCollection()->getFirstItem();
            
                if (empty($historyItem))
                {
                    return;
                }

                $comment = $historyItem->getComment();

                if (empty($comment))
                {
                    return;
                }

                $tempStrpos = strpos($comment, \YottaPayIntegration\Payments\Model\Common::$historyCommentPaymentIntent);

                if ($lastRealOrder->getData('state') === 'pending_payment' && $tempStrpos !== false)
                {
                    $this->_logger->error('YottaPay | CheckBackButton execute | Restore quote');
                    $this->_checkoutSession->restoreQuote();
                }
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | CheckBackButton execute | ' . $e->getMessage());
        }
    }
}
